#ifndef WIN32
#include <windows.h>
#endif
#include "wresourc.h"

InputDlgBox DIALOG 64, 32, 150, 57
STYLE DS_MODALFRAME | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "Input"
BEGIN
    CONTROL "",        ID_PROMPT, "static", SS_LEFT | WS_GROUP | WS_CHILD,                   6,  5, 138, 10
    CONTROL "",        ID_ANSWER, "edit",   ES_LEFT | ES_AUTOHSCROLL | WS_BORDER | WS_TABSTOP | WS_CHILD,    6, 17, 138, 12
    CONTROL "&Ok",     IDOK,      "button", BS_DEFPUSHBUTTON | WS_GROUP | WS_TABSTOP | WS_CHILD,        30, 36,  30, 15
    CONTROL "&Cancel", IDCANCEL,  "button", BS_PUSHBUTTON | WS_GROUP | WS_TABSTOP | WS_CHILD,           90, 36,  30, 15
END

PrintDlgBox DIALOG 40, 40, 120, 40
STYLE WS_POPUP | WS_CAPTION | WS_SYSMENU | WS_VISIBLE
BEGIN
	CTEXT			"Cancel Printing", -1, 4, 6, 120, 12
	DEFPUSHBUTTON	"Cancel", IDCANCEL, 44, 22, 32, 14, WS_GROUP
END

PrintSizeDlgBox DIALOG 16, 32, 210, 96
STYLE DS_MODALFRAME | WS_POPUP | WS_CAPTION | WS_SYSMENU | WS_VISIBLE
CAPTION "Print Size"
BEGIN
	DEFPUSHBUTTON "OK", IDOK, 168, 16, 33, 14, WS_CHILD | WS_VISIBLE | WS_TABSTOP
	PUSHBUTTON "Cancel", IDCANCEL, 168, 40, 34, 14
	CONTROL "Size", PSIZE_SBOX, "BUTTON", BS_GROUPBOX | WS_CHILD | WS_VISIBLE | WS_GROUP, 6, 2, 153, 58
	CONTROL "&Default Size", PSIZE_DEF, "BUTTON", BS_AUTORADIOBUTTON | WS_CHILD | WS_VISIBLE | WS_TABSTOP, 12, 11, 52, 10
	CONTROL "&Other Size", PSIZE_OTHER, "BUTTON", BS_AUTORADIOBUTTON | WS_CHILD | WS_VISIBLE | WS_TABSTOP, 12, 34, 52, 10
	RTEXT "Width", -1, 30, 23, 24, 8, SS_RIGHT | WS_CHILD | WS_VISIBLE | WS_GROUP
	CONTROL "", PSIZE_DEFX, "STATIC", SS_LEFT | WS_CHILD | WS_VISIBLE, 57, 23, 16, 8
	LTEXT "mm", -1, 74, 23, 12, 8
	RTEXT "Height", -1, 97, 23, 24, 8, SS_RIGHT | WS_CHILD | WS_VISIBLE | WS_GROUP
	CONTROL "", PSIZE_DEFY, "STATIC", SS_LEFT | WS_CHILD | WS_VISIBLE, 125, 23, 16, 8
	LTEXT "mm", -1, 142, 23, 12, 8
	RTEXT "Width", -1, 30, 46, 24, 8, SS_RIGHT | WS_CHILD | WS_VISIBLE | WS_GROUP
	CONTROL "120", PSIZE_X, "EDIT", ES_LEFT | WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP, 56, 44, 16, 12
	LTEXT "mm", -1, 74, 46, 12, 8
	LTEXT "Height", -1, 100, 46, 24, 8
	CONTROL "90", PSIZE_Y, "EDIT", ES_LEFT | WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP, 124, 44, 16, 12
	LTEXT "mm", -1, 142, 46, 12, 8
	CONTROL "Page Offset", PSIZE_OFFBOX, "BUTTON", BS_GROUPBOX | WS_CHILD | WS_VISIBLE | WS_GROUP, 6, 64, 153, 27
	RTEXT "Left", -1, 29, 76, 24, 8, SS_RIGHT | WS_CHILD | WS_VISIBLE | WS_GROUP
	CONTROL "0", PSIZE_OFFX, "EDIT", ES_LEFT | WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP, 56, 74, 16, 12
	LTEXT "mm", -1, 74, 76, 12, 8
	RTEXT "Top", -1, 100, 76, 21, 8, SS_RIGHT | WS_CHILD | WS_VISIBLE | WS_GROUP
	CONTROL "0", PSIZE_OFFY, "EDIT", ES_LEFT | WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP, 124, 74, 16, 12
	LTEXT "mm", -1, 142, 76, 12, 8
END

LineStyleDlgBox DIALOG 16, 32, 172, 137
STYLE DS_MODALFRAME | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "Line Styles"
BEGIN
	CONTROL "", LS_LINENUM, "LISTBOX", LBS_NOTIFY | WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL | WS_TABSTOP, 8, 8, 49, 47
	CONTROL "Monochrome", LS_MONOBOX, "button", BS_GROUPBOX | WS_CHILD | WS_VISIBLE, 72, 4, 92, 52
	CONTROL "Color", LS_COLORBOX, "button", BS_GROUPBOX | WS_CHILD | WS_VISIBLE, 8, 60, 156, 52
	LTEXT "Style", -1, 80, 20, 22, 8, WS_CHILD | WS_VISIBLE | WS_GROUP
	CONTROL "", LS_MONOSTYLE, "COMBOBOX", CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE | WS_VSCROLL | WS_TABSTOP, 106, 18, 52, 52
	LTEXT "Width", -1, 80, 40, 20, 8, WS_CHILD | WS_VISIBLE | WS_GROUP
	EDITTEXT LS_MONOWIDTH, 106, 38, 16, 12
	PUSHBUTTON "&Choose Color", LS_CHOOSECOLOR, 16, 92, 52, 14, WS_CHILD | WS_VISIBLE | WS_TABSTOP
	CONTROL "", LS_COLORSAMPLE, "BUTTON", BS_OWNERDRAW | WS_CHILD | WS_VISIBLE, 36, 75, 12, 8
	LTEXT "Style", -1, 80, 76, 20, 8, WS_CHILD | WS_VISIBLE | WS_GROUP
	CONTROL "", LS_COLORSTYLE, "COMBOBOX", CBS_DROPDOWNLIST | WS_CHILD | WS_VISIBLE | WS_VSCROLL | WS_TABSTOP, 106, 74, 52, 52
	LTEXT "Width", -1, 80, 96, 19, 8, WS_CHILD | WS_VISIBLE | WS_GROUP
	EDITTEXT LS_COLORWIDTH, 106, 94, 16, 12
	PUSHBUTTON "OK", IDOK, 16, 118, 32, 14, WS_CHILD | WS_VISIBLE | WS_TABSTOP
	PUSHBUTTON "Cancel", IDCANCEL, 70, 118, 32, 14, WS_CHILD | WS_VISIBLE | WS_TABSTOP
	PUSHBUTTON "&Default", LS_DEFAULT, 124, 118, 32, 14, WS_CHILD | WS_VISIBLE | WS_TABSTOP
END

AboutDlgBox DIALOG 32, 32, 180, 80
STYLE DS_MODALFRAME | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "About"
BEGIN
    CONTROL "", AB_ICON, "BUTTON", BS_OWNERDRAW | WS_CHILD | WS_VISIBLE, 4, 4, 16, 16
    LTEXT   "", AB_TEXT1,  24,  8, 96, 10
    PUSHBUTTON "OK", IDOK,  142, 6,  32, 12
    LTEXT   "", AB_TEXT2,  4, 22, 172, 48
    LTEXT   "", AB_TEXT3,  4, 70, 172, 48
END

