#define DIALOGS_H
#define IDM_HELPINDEX         10004
#define IDM_KEYSHELP          10003
#define IDM_EXTENDEDHELP      10002
#define IDM_HELPFORHELP       10001
#define IDM_FILE              100
#define IDM_PRINTSETUP        115
#define IDM_PRINTPIC          111
#define IDM_PRINT             101
#define IDM_EXIT              102
#define IDM_ABOUT             103
#define IDM_FONTS             104
#define IDM_SAVE              105
#define IDM_COMMAND           300
#define IDM_CONTINUE          400
#define IDM_PAUSEOPT          120
#define IDM_PAUSEDLG          121
#define IDM_PAUSEBTN          122
#define IDM_PAUSEGNU          123
#define IDM_OPTIONS           200
#define IDM_OPTIONMAIN        201
#define IDM_COLOURS           206
#define IDM_LINES             207
#define IDM_LINES_THICK       208
#define IDM_LINES_SOLID       209
#define ID_ABOUT              10
#define IDD_COLOURS           20
#define IDD_PRINTQNAME        5106
#define IDD_PRINTSETUP        5105
#define IDD_PRINTPIC          5101
#define ID_PRINT              5100
#define ID_QPRINT             5000
#define ID_QPRINTERS          5200
#define IDD_QPRSLIST          5201
#define IDD_QPRSSEL           5202
#define IDD_QPRNAME           5009
#define IDD_QPRFRAME          5008
#define IDD_QPRTRACK          5007
#define IDD_QPRSETPR          5006
#define IDD_QPRXSIZE          5001
#define IDD_QPRYSIZE          5002
#define IDD_QPRXFRAC          5003
#define IDD_QPRYFRAC          5004
#define IDD_QPRBOX            5005
#define IDD_FONTS             6000
#define IDD_PAUSEBOX          3000
#define IDD_PAUSETEXT         3001

#define IDD_QUERYPRINT              2000
#define IDD_QPTEXT                  2001
#define IDD_QPLISTBOX               2002
#define IDD_QPJOBPROP               2005


//#define DID_HELP     10
//#define DID_CANCEL   2
//#define DID_OK       1
#define IDH_EXTENDED          905
#define IDH_INDEX             904
#define IDH_KEYS              903
#define IDH_FOREXTENDED       902
#define IDH_FORHELP           901

#define IDD_PRINTNAME               5010
