#include <os2.h>
#include "dialogs.h"

ICON 1 "gnuplot.ico"


MENU 1
BEGIN
    SUBMENU  "~Options" , IDM_FILE
    BEGIN
        MENUITEM    "Prin~ters...", IDM_PRINTSETUP
        MENUITEM    "~Print..."   , IDM_PRINT
        MENUITEM    "~Fonts...", IDM_FONTS
        MENUITEM    "~Colours",  IDM_COLOURS,,MIA_CHECKED
        SUBMENU     "~Lines", IDM_LINES
        BEGIN
            MENUITEM  "~Solid", IDM_LINES_SOLID,,MIA_CHECKED
            MENUITEM  "~Thick", IDM_LINES_THICK
        END
        SUBMENU     "Pa~use mode", IDM_PAUSEOPT
        BEGIN
            MENUITEM  "~Dialog box", IDM_PAUSEDLG,,MIA_CHECKED
            MENUITEM  "~Menu bar", IDM_PAUSEBTN
            MENUITEM  "~Gnuplot", IDM_PAUSEGNU
        END
        MENUITEM  SEPARATOR
        MENUITEM    "~Save settings", IDM_SAVE
    END
    MENUITEM "~Gnuplot" , IDM_COMMAND
    MENUITEM "~Continue" , IDM_CONTINUE,,MIA_DISABLED
    SUBMENU "~Help", 1
    BEGIN
        MENUITEM "~Help for Help...",    IDM_HELPFORHELP
        MENUITEM "~Extended Help...",    IDM_EXTENDEDHELP
        MENUITEM "~Keys Help...",        IDM_KEYSHELP
        MENUITEM "Help ~index...",       IDM_HELPINDEX
        MENUITEM SEPARATOR
        MENUITEM "A~bout...",            IDM_ABOUT
    END
END

ACCELTABLE 1
BEGIN
    VK_ESC, IDM_COMMAND, VIRTUALKEY 
END

DLGTEMPLATE ID_ABOUT LOADONCALL MOVEABLE DISCARDABLE
BEGIN
    DIALOG  "", ID_ABOUT, 22, 17, 144, 75, 
    BEGIN
        CTEXT           "GnuplotPM", -1, 8, 65, 128, 8
        CTEXT           "Gnuplot display for OS/2", -1, 8, 50, 128, 8
        CTEXT           "$Revision: 3.38.2.59 $", -1, 8, 35, 128, 8
        DEFPUSHBUTTON   "OK", DID_OK, 56, 10, 32, 14, WS_GROUP
    END
END

DLGTEMPLATE ID_QPRINT LOADONCALL MOVEABLE DISCARDABLE
BEGIN
    DIALOG  "GnuplotPM: printer setup", ID_QPRINT, 83, 7, 220, 175, 
            FS_NOBYTEALIGN | WS_VISIBLE, FCF_TITLEBAR
    BEGIN
        DEFPUSHBUTTON   "OK", DID_OK, 6, 5, 38, 13
        PUSHBUTTON      "Cancel", DID_CANCEL, 52, 5, 38, 13
        PUSHBUTTON      "Help", 310, 176, 5, 38, 13, BS_HELP
        GROUPBOX        "Set plot area", -1, 2, 41, 214, 116
        PUSHBUTTON      "Adjust...", IDD_QPRTRACK, 7, 111, 56, 13
        CONTROL         "Text", IDD_QPRBOX, 84, 96, 98, 49, WC_STATIC, 
                        SS_FGNDRECT | WS_VISIBLE
        CONTROL         "", IDD_QPRFRAME, 86, 98, 71, 34, WC_STATIC, 
                        SS_HALFTONERECT | WS_VISIBLE
        ENTRYFIELD      "Text", IDD_QPRXSIZE, 83, 47, 36, 12, ES_RIGHT | NOT 
                        ES_AUTOSCROLL | ES_READONLY
        ENTRYFIELD      "", IDD_QPRXFRAC, 168, 47, 39, 8, NOT ES_AUTOSCROLL | 
                        ES_MARGIN
        LTEXT           "Vertical size:", -1, 8, 63, 66, 8, NOT WS_GROUP
        LTEXT           "Horizontal size:", -1, 8, 47, 70, 8, NOT WS_GROUP
        ENTRYFIELD      "Text", IDD_QPRYSIZE, 83, 63, 36, 12, ES_RIGHT | NOT 
                        ES_AUTOSCROLL | ES_READONLY
        ENTRYFIELD      "", IDD_QPRYFRAC, 168, 63, 39, 8, NOT ES_AUTOSCROLL | 
                        ES_MARGIN
        LTEXT           "Or enter fraction of indicated size:", -1, 9, 79, 
                        143, 8, NOT WS_GROUP
        LTEXT           "cm", -1, 125, 63, 13, 8, NOT WS_GROUP
        LTEXT           "cm", -1, 125, 47, 13, 8
        PUSHBUTTON      "Job properties...", IDD_QPRSETPR, 98, 5, 72, 13
        LTEXT           "Print to file named:", IDD_PRINTQNAME, 8, 28, 80, 8, 
                        DT_HALFTONE
        ENTRYFIELD      "", IDD_QPRNAME, 91, 28, 87, 8, NOT ES_AUTOSCROLL | 
                        ES_MARGIN
        LTEXT           "Adjust using", -1, 8, 136, 72, 8
        LTEXT           "mouse...", -1, 8, 127, 37, 8
        LTEXT           "Current printer:", -1, 8, 161, 64, 8
        ENTRYFIELD      "", IDD_PRINTNAME, 74, 161, 139, 8, ES_MARGIN
    END
END

DLGTEMPLATE IDD_QUERYPRINT LOADONCALL MOVEABLE DISCARDABLE
BEGIN
    DIALOG  "GnuplotPM: printer selection", IDD_QUERYPRINT, 59, 44, 243, 99, 
            FS_NOBYTEALIGN | WS_VISIBLE, FCF_SYSMENU | FCF_TITLEBAR
    BEGIN
        LTEXT           "Select printer:", IDD_QPTEXT, 4, 89, 65, 8
        LISTBOX         IDD_QPRSLIST, 5, 29, 232, 60
        DEFPUSHBUTTON   "OK", DID_OK, 5, 5, 38, 13, WS_GROUP
        PUSHBUTTON      "Cancel", DID_CANCEL, 54, 5, 38, 13
        PUSHBUTTON      "Help", 310, 102, 5, 38, 13, BS_HELP
    END
END

DLGTEMPLATE IDD_PAUSEBOX LOADONCALL MOVEABLE DISCARDABLE
BEGIN
    DIALOG  "Gnuplot: pause", IDD_PAUSEBOX, 69, 27, 148, 33, WS_VISIBLE, 
            FCF_TITLEBAR | FCF_MINBUTTON
    BEGIN
        PUSHBUTTON      "Continue", DID_OK, 1, 0, 40, 14
        PUSHBUTTON      "Cancel", DID_CANCEL, 54, 0, 40, 14
        PUSHBUTTON      "Help", 310, 106, 0, 40, 14, BS_HELP
        LTEXT           "Text", IDD_PAUSETEXT, 0, 20, 148, 8
    END
END

HELPSUBTABLE 1
BEGIN
    HELPSUBITEM  IDH_EXTENDED, IDH_EXTENDED
    HELPSUBITEM  IDM_PRINTSETUP ,IDM_PRINTSETUP 
    HELPSUBITEM  IDM_PRINT ,IDM_PRINT 
    HELPSUBITEM  IDM_FONTS ,IDM_FONTS 
    HELPSUBITEM  IDM_FILE ,IDM_FILE 
    HELPSUBITEM  IDM_ABOUT ,IDM_ABOUT 
    HELPSUBITEM  IDM_COLOURS ,IDM_COLOURS 
    HELPSUBITEM  IDM_LINES ,IDM_LINES 
    HELPSUBITEM  IDM_LINES_THICK ,IDM_LINES_THICK 
    HELPSUBITEM  IDM_LINES_SOLID ,IDM_LINES_SOLID
    HELPSUBITEM  IDM_SAVE ,IDM_SAVE 
    HELPSUBITEM  IDM_COMMAND ,IDM_COMMAND  
    HELPSUBITEM  IDM_CONTINUE,IDM_CONTINUE
    HELPSUBITEM  IDM_PAUSEDLG,IDM_PAUSEDLG 
    HELPSUBITEM  IDM_PAUSEOPT,IDM_PAUSEOPT 
    HELPSUBITEM  IDM_PAUSEBTN,IDM_PAUSEBTN 
    HELPSUBITEM  IDM_PAUSEGNU,IDM_PAUSEGNU 
END
HELPSUBTABLE ID_PRINT
BEGIN
    HELPSUBITEM  ID_PRINT, ID_PRINT
END
HELPSUBTABLE ID_QPRINT
BEGIN
    HELPSUBITEM  ID_QPRINT, ID_QPRINT
END
HELPSUBTABLE IDD_QUERYPRINT
BEGIN
    HELPSUBITEM  IDD_QUERYPRINT, IDD_QUERYPRINT
END
HELPSUBTABLE IDD_PAUSEBOX
BEGIN
    HELPSUBITEM  IDD_PAUSEBOX, IDD_PAUSEBOX
END
HELPTABLE 1
BEGIN
    HELPITEM 1, 1, IDH_EXTENDED
    HELPITEM ID_QPRINT, ID_QPRINT, ID_QPRINT
    HELPITEM ID_PRINT, ID_PRINT, ID_PRINT
    HELPITEM IDD_QUERYPRINT, IDD_QUERYPRINT, IDD_QUERYPRINT
    HELPITEM IDD_PAUSEBOX, IDD_PAUSEBOX, IDD_PAUSEBOX
END


